<?php
namespace Controller;

use \Service\Server as ServerService;
use Zeedhi\DevOps\Util\Utilities;
use Zeedhi\Framework\Controller\Simple;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request\DataSet as RequestDataset;
use Zeedhi\Framework\DTO\Request\Filter as RequestFilter;
use Zeedhi\Framework\DTO\Request\Row as RequestRow;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\Log\Exception;

class Server extends \Generated\Controller\Server {
    protected $serverService;

    /**
     * @param ServerService $serverService
     */
    public function __construct(ServerService $serverService) {
        $this->serverService = $serverService;
    }

    public function findMnemonicByServer(RequestFilter $request, Response $response) {
        $conditions = $request->getFilterCriteria()->getConditions();
        $serverId = Utilities::getValueInFilterDataSourceByName($conditions,'SERVER_ID', true);
        $productsId = Utilities::getValueInFilterDataSourceByName($conditions,'PRODUCTS_ID', true);
        if (empty($productsId)){
            $filterWhereInByProductsId = "";
        } else {
            $filterWhereInByProductsId = "AND ". Utilities::fixBugWhereINQuery($productsId, "MN.PRODUCT_ID");
        }
        $filters = Utilities::createQueryFiltered($conditions);
        $page = $request->getFilterCriteria()->getPage();
        $pageSize = $request->getFilterCriteria()->getPageSize();
        $productFiles = $this->serverService->findMnemonicByServer($serverId, $filters, $filterWhereInByProductsId, $page, $pageSize);
        $response->addDataSet(new DataSet('findMnemonicByProductFiles', $productFiles));
    }
    
    public function findProductsByServer(RequestFilter $request, Response $response){
        $conditions = $request->getFilterCriteria()->getConditions();
        $serverId = Utilities::getValueInFilterDataSourceByName($conditions,'SERVER_ID', true);
        $products = $this->serverService->findProductsByServer($serverId);
        $response->addDataSet(new DataSet('products', $products));
    }

    public function mnemonicByServerSave(RequestDataset $request, Response $response)
    {
        try {
            $rows = $request->getDataSet()->getRows();
            $row = $rows[0];
            $mnemonicId = $row['MNEMONIC_ID'];
            $mnemonicString = $row['MNEMONIC'];
            $serverId = $row['SERVER_ID'];
            $value = $row['VALUE'];
            $label = $row['LABEL'];
            $isEncrypted = $row['IS_ENCRYPTED'];
            $mnemonicGrouping =  empty($row['MNEMONIC_GROUP_ID']) ? null : $row['MNEMONIC_GROUP_ID'];
            $productFiles = $this->serverService->saveMnemonicByServer($serverId, $mnemonicId, $mnemonicString, $value, $label, $isEncrypted, $mnemonicGrouping);
            $response->addDataSet(new DataSet('findMnemonicByProductFiles', [$productFiles]));
        } catch (\Exception $ex) {
            $response->addMessage(new Response\Message($ex->getMessage()));
            $response->setError(new Response\Error($ex->getMessage(), $ex->getCode()));
        }
    }

    public function mnemonicByServerRemove(\Zeedhi\Framework\DTO\Request $request, Response $response)
    {
        $rows = $request->getDataSet()->getRows();
        $row = $rows[0];
        $mnemonicId = $row['MNEMONIC_ID'];
        $serverId = $row['SERVER_ID'];
        $this->serverService->removeMnemonicByServer($mnemonicId, $serverId);
    }

    /**
     * @param RequestDataset $request
     * @param Response $response
     */
    public function saveServer(RequestDataset $request, Response $response) {
        try {
            $serverData      = $request->getDataSet()->getRows();
            $serverData      = $serverData[0];
            $productServerId = isset($serverData['ID']) ? $serverData['ID'] : null;
            $productId       = isset($serverData['PRODUCT_ID']) ? $serverData['PRODUCT_ID'] : null;
            $useType         = isset($serverData['USETYPE']) ? $serverData['USETYPE'] : null;
            $serverId        = isset($serverData['SERVER_ID']) ? $serverData['SERVER_ID'] : null;
            $name            = $serverData['NAME'];
            $wsEmail         = $serverData['WSEMAIL'];
            $id              = $serverData['ID'];
            $onlyApproved    = isset($serverData['ONLY_APPROVED_SCRIPTS']) ? $serverData['ONLY_APPROVED_SCRIPTS'] : 'Y';
            $description     = isset($serverData['DESCRIPTION']) ? $serverData['DESCRIPTION'] : null;
            $baseUrl         = isset($serverData['BASE_URL']) ? $serverData['BASE_URL'] : null;
            $basePath        = isset($serverData['PATH']) ? $serverData['PATH'] : null;
            $backupLimit     = isset($serverData['BACKUP_LIMIT']) ? $serverData['BACKUP_LIMIT'] : null;
            $isActive        = isset($serverData['IS_ACTIVE']) ? $serverData['IS_ACTIVE'] : 'Y';
            $resourceId      = isset($serverData['RESOURCE_T_ID']) ? $serverData['RESOURCE_T_ID'] : null;
            $isProductServer = isset($serverData['isProductServer']) ? $serverData['isProductServer'] : null;
            $isWindows       = isset($serverData['IS_WINDOWS']) ? $serverData['IS_WINDOWS'] : null;
            $isProduction       = isset($serverData['IS_PRODUCTION']) ? $serverData['IS_PRODUCTION'] : null;

            $serverId = $this->serverService->insertOrUpdateServer($productServerId, $serverId, $productId, $name, $wsEmail, $useType,
                $description, $baseUrl, $basePath, $id, $backupLimit, $onlyApproved, $isActive, $resourceId, $isProductServer, $isWindows, $isProduction);
            $response->addDataSet(new DataSet('server', [$serverId]));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }

    public function saveServerForEach(RequestDataset $request, Response $response) {
        try {
            $serverData = $request->getDataSet()->getRows();
            $serverData = $serverData[0];
            $servers = $this->serverService->insertOrUpdateServerForEach($serverData);
            $response->addDataSet(new DataSet('server', $servers));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    /**
     * @param RequestRow $request
     * @param Response $response
     */
    public function removeServer(RequestRow $request, Response $response) {
        $server = $request->getRow();
        try {
            $products = $this->serverService->removeServer($server['ID']);
            $response->addDataSet(new DataSet('removeServer', $products));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage()));
        }
    }

    /**
     * @param RequestFilter $request
     * @param Response $response
     */
    public function serverByProduct(RequestFilter $request, Response $response){
        $conditions = $request->getFilterCriteria()->getConditions();
        $page = $request->getFilterCriteria()->getPage();
        $pageSize = $request->getFilterCriteria()->getPageSize();
        $filters = Utilities::createQueryFiltered($conditions);
        try {
            $servers = $this->serverService->findServersWithFilter($filters, $page, $pageSize);
            $response->addDataSet(new DataSet('server', $servers));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }

    /**
     * @param RequestFilter $request
     * @param Response $response
     */
    public function findAllServers(RequestFilter $request, Response $response){
        try {
            $conditions = $request->getFilterCriteria()->getConditions();
            $page = $request->getFilterCriteria()->getPage();
            $pageSize = $request->getFilterCriteria()->getPageSize();
            $filters = Utilities::createQueryFiltered($conditions);
            $servers = $this->serverService->findServers($filters, $page, $pageSize);
            $response->addDataSet(new DataSet('server', $servers));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }

    /**
     * @param RequestRow $request
     * @param Response $response
     */
    public function removeProductServer(RequestRow $request, Response $response) {
        $server = $request->getRow();
        try {
            $this->serverService->removeProductServer($server['ID']);
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage()));
        }
    }

    /**
     * @param RequestFilter $request
     * @param Response $response
     */
    public function serverByProductAndType(RequestFilter $request, Response $response){
        $conditions = $request->getFilterCriteria()->getConditions();
        $productId  = Utilities::getValueInFilterDataSourceByName($conditions,'PRODUCT_ID');
        $type       = Utilities::getValueInFilterDataSourceByName($conditions,'TYPE');
        $filterProductServerStatus    = Utilities::getValueInFilterDataSourceByName($conditions,'PRODUCT_SERVER_IS_ACTIVE');
        
        try {
            $servers = $this->serverService->serverByProductAndType($productId, $type, $filterProductServerStatus);
            $response->addDataSet(new DataSet('server', $servers));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }

    public function validateSubReleaseServerPermissions(RequestFilter $request, Response $response){
        try{
            $conditions = $request->getFilterCriteria()->getConditions();
            $subReleaseId = Utilities::getValueInFilterDataSourceByName($conditions,'SUBRELEASE_ID');
            $operation = Utilities::getValueInFilterDataSourceByName($conditions,'OPERATION');
            $needsApproval = $this->serverService->validateSubReleaseServerPermissions($subReleaseId, $operation);
            $response->addDataSet(new DataSet('validateSubReleaseServerPermissions', $needsApproval));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }

    public function validateReleaseServerPermissions(RequestFilter $request, Response $response){
        try{
            $conditions = $request->getFilterCriteria()->getConditions();
            $releaseId = Utilities::getValueInFilterDataSourceByName($conditions,'RELEASE_ID');
            $operation = Utilities::getValueInFilterDataSourceByName($conditions,'OPERATION');
            $needsApproval = $this->serverService->validateReleaseServerPermissions($releaseId, $operation);
            $response->addDataSet(new DataSet('validateReleaseServerPermissions', $needsApproval));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }
    
    public function copyMnemonic(RequestFilter $request, Response $response){
        try{
            $conditions = $request->getFilterCriteria()->getConditions();
            $originalId = Utilities::getValueInFilterDataSourceByName($conditions,'EXPORTED_SERVER_ID');
            $exportedServerRows = Utilities::getValueInFilterDataSourceByName($conditions,'EXPORTED_SERVER_ROWS');
            $copyMnemonics = $this->serverService->findMnemonicToExport($originalId, $exportedServerRows);
            $response->addDataSet(new DataSet('copyMnemonic', [$copyMnemonics]));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }

    public function validateDeployServerPermissions(RequestFilter $request, Response $response){
        $conditions = $request->getFilterCriteria()->getConditions();
        $deployId = Utilities::getValueInFilterDataSourceByName($conditions,'DEPLOY_ID');
        $operation = Utilities::getValueInFilterDataSourceByName($conditions,'OPERATION');
        $server = Utilities::getValueInFilterDataSourceByName($conditions,'SERVER');
        $needsApproval = $this->serverService->validateDeployServerPermissions($deployId, $operation, $server);
        $response->addDataSet(new DataSet('validateDeployServerPermissions', $needsApproval));
    }
}

